// grapheditView.cpp : implementation of the CGrapheditView class
//

#include "stdafx.h"
#include "graphedit.h"

#include "grapheditDoc.h"
#include "grapheditView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void CGrapheditView::drag_arc()
{
	CDC * dc = GetDC();
	int    old_mode = dc->SetROP2(R2_XORPEN);
	CPen * old_pen  = (CPen *)dc->SelectStockObject( WHITE_PEN );
	
	dc->MoveTo( start_drag );
	dc->LineTo( end_drag );

 	dc->SelectObject( old_pen );
	dc->SetROP2(old_mode);
	ReleaseDC(dc);
}

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView

IMPLEMENT_DYNCREATE(CGrapheditView, CView)

BEGIN_MESSAGE_MAP(CGrapheditView, CView)
	//{{AFX_MSG_MAP(CGrapheditView)
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_MODE_INSERT_N, OnModeInsertN)
	ON_UPDATE_COMMAND_UI(ID_MODE_INSERT_N, OnUpdateModeInsertN)
	ON_COMMAND(ID_MODE_INSERT_ARC, OnModeInsertArc)
	ON_UPDATE_COMMAND_UI(ID_MODE_INSERT_ARC, OnUpdateModeInsertArc)
	ON_COMMAND(ID_MODE_EDIT, OnModeEdit)
	ON_UPDATE_COMMAND_UI(ID_MODE_EDIT, OnUpdateModeEdit)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView construction/destruction

CGrapheditView::CGrapheditView()
{
  state      = S_INSERT_NODE;
  start_node = -1;
  captured   = false;
}

CGrapheditView::~CGrapheditView()
{
}

BOOL CGrapheditView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView drawing

void CGrapheditView::OnDraw(CDC* pDC)
{
	const int NODE_R = 4;

	CGrapheditDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CGrapheditDoc::arc_iterator a;

	CPen apen(PS_SOLID,3,RGB(0,0,128));

	CPen * old_pen = (CPen *)(pDC->SelectObject(&apen));

	for(a=pDoc->arcs.begin();a!=pDoc->arcs.end();++a)
	{
		pDC->MoveTo( pDoc->nodes[a->n[0]].p );
		pDC->LineTo( pDoc->nodes[a->n[1]].p );
	}

	CPen npen(PS_SOLID,2,RGB(128,0,0));

	CBrush nbrush( RGB(255,255,0) );

	pDC->SelectObject(&npen);

	CBrush * old_brush = (CBrush *)(pDC->SelectObject(&nbrush));

	CGrapheditDoc::node_iterator n;

	for(n=pDoc->nodes.begin();n!=pDoc->nodes.end();++n)
	{
		pDC->Ellipse(
			n->p.x-NODE_R,
			n->p.y-NODE_R,
			n->p.x+NODE_R,
			n->p.y+NODE_R
		);
	}

	pDC->SelectObject(old_pen);
	pDC->SelectObject(old_brush);
}

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView printing

BOOL CGrapheditView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CGrapheditView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CGrapheditView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView diagnostics

#ifdef _DEBUG
void CGrapheditView::AssertValid() const
{
	CView::AssertValid();
}

void CGrapheditView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGrapheditDoc* CGrapheditView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGrapheditDoc)));
	return (CGrapheditDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGrapheditView message handlers

void CGrapheditView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CGrapheditDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	switch(state)
	{
	case S_INSERT_NODE:
		pDoc->add_node(point);
		break;
	case S_INSERT_ARC:
		{
			start_node = pDoc->FindNode(point);
			if(start_node!=-1)
			{
				end_node = start_node;
				start_drag = end_drag = pDoc->nodes[start_node].p;  
				drag_arc();
				SetCapture();
				captured = true;
			}
		}
		break;
	}
}

void CGrapheditView::OnModeInsertN() 
{
	state = S_INSERT_NODE;	
}

void CGrapheditView::OnUpdateModeInsertN(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
	pCmdUI->SetCheck(state == S_INSERT_NODE);
}

void CGrapheditView::OnModeInsertArc() 
{
	state = S_INSERT_ARC;
}

void CGrapheditView::OnUpdateModeInsertArc(CCmdUI* pCmdUI) 
{
	CGrapheditDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pCmdUI->Enable(pDoc->nodes.size()>1);
	pCmdUI->SetCheck(state == S_INSERT_ARC);
}

void CGrapheditView::OnModeEdit() 
{
	state = S_EDIT;
}

void CGrapheditView::OnUpdateModeEdit(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
	pCmdUI->SetCheck(state == S_EDIT);
}

void CGrapheditView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(captured)
	{
		drag_arc();
		end_drag = point;
		drag_arc();
	}
}

void CGrapheditView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if(captured)
	{
		CGrapheditDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
			
		drag_arc();
		ReleaseCapture();
		captured = false;
		end_node = pDoc->FindNode(point);
		if( end_node!=-1 && end_node!=start_node)
		{
			pDoc->add_arc(start_node,end_node);
		}
	}
}
